import tkinter as tk
from tkinter import filedialog, messagebox
import paramiko
import os
import tempfile
from pydub import AudioSegment
import speech_recognition as sr
import anthropic
 
def consultar_claude(text):
    client = anthropic.Anthropic(api_key='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')
    message = client.messages.create(
        model="claude-3-5-haiku-20241022",
        max_tokens=1000,
        temperature=0,
        system=(
            "A continuación te paso el texto de una llamada y necesito un resumen que incluya: necesidad del cliente, estado de ánimo y la solución brindada."
        ),
        messages=[
            {
                "role": "user",
                "content": [{"type": "text", "text": text}]
            }
        ]
    )
    respuesta_claude = "".join([msg.text for msg in message.content])
    return respuesta_claude
 
def connect_and_download(ip, key, audio_path, local_path):
    ssh = paramiko.SSHClient()
    ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    try:
        ssh.connect(ip, username='root', password=key)
        sftp = ssh.open_sftp()
        sftp.get(audio_path, local_path)
        sftp.close()
        ssh.close()
        return True
    except Exception as e:
        print(f"Error: {e}")
        return False
 
def split_audio(file_path, output_folder):
    audio = AudioSegment.from_file(file_path)
    duration = len(audio)  # Duration in milliseconds
    chunk_duration = 60 * 1000  # 1 minute in milliseconds
    chunks = []
    for i in range(0, duration, chunk_duration):
        chunk = audio[i:i + chunk_duration]
        chunk_name = os.path.join(output_folder, f"chunk_{i//chunk_duration + 1}.wav")
        chunk.export(chunk_name, format="wav")
        chunks.append(chunk_name)
    return chunks
 
def transcribe_audio_chunks(chunks, output_file):
    recognizer = sr.Recognizer()
    with open(output_file, "w") as file:
        for idx, chunk in enumerate(chunks, 1):
            with sr.AudioFile(chunk) as source:
                audio_data = recognizer.record(source)
                try:
                    text = recognizer.recognize_google(audio_data, language="es-ES")
                    file.write(f"Fragmento {idx}: {text}\n")
                except sr.UnknownValueError:
                    file.write(f"Fragmento {idx}: No se pudo reconocer el audio\n")
                except sr.RequestError as e:
                    file.write(f"Fragmento {idx}: Error en el servicio de reconocimiento ({e})\n")
 
def main():
    ip = ip_entry.get()
    key = key_entry.get()
    audio_path = audio_path_entry.get()
    # Usar ruta temporal
    temp_dir = tempfile.gettempdir()
    local_path = os.path.join(temp_dir, "downloaded_audio.mp3")
    output_folder = os.path.join(temp_dir, "audio_chunks")
    os.makedirs(output_folder, exist_ok=True)
    if connect_and_download(ip, key, audio_path, local_path):
        chunks = split_audio(local_path, output_folder)
        resumen_path = os.path.join(temp_dir, "resumen_llamada.txt")
        transcribe_audio_chunks(chunks, resumen_path)
        # Leer el contenido para procesar con Claude
        with open(resumen_path, "r") as file:
            contenido = file.read()
        respuesta_claude = consultar_claude(contenido)
        # Mostrar en la interfaz
        resumen_text.delete("1.0", tk.END)
        resumen_text.insert(tk.END, respuesta_claude)
        # Guardar la respuesta
        global respuesta_final
        respuesta_final = respuesta_claude
        status_label.config(text="Proceso completado. Resumen disponible.")
    else:
        status_label.config(text="Error en la descarga o conexión.")
 
def save_summary():
    save_path = filedialog.asksaveasfilename(defaultextension=".txt", filetypes=[("Text Files", "*.txt")])
    if save_path:
        with open(save_path, "w") as file:
            file.write(respuesta_final)
        messagebox.showinfo("Guardar Resumen", "El resumen ha sido guardado exitosamente.")
 
# Interfaz gráfica con tkinter
app = tk.Tk()
app.title("Audio Downloader and Transcriber")
 
tk.Label(app, text="IP del Servidor:").grid(row=0, column=0)
ip_entry = tk.Entry(app)
ip_entry.grid(row=0, column=1)
 
tk.Label(app, text="Clave del Servidor:").grid(row=1, column=0)
key_entry = tk.Entry(app, show="*")
key_entry.grid(row=1, column=1)
 
tk.Label(app, text="Ruta del Audio en el Servidor:").grid(row=2, column=0)
audio_path_entry = tk.Entry(app)
audio_path_entry.grid(row=2, column=1)
 
download_button = tk.Button(app, text="Descargar y Procesar", command=main)
download_button.grid(row=3, column=0, columnspan=2)
 
tk.Label(app, text="Resumen de la Llamada:").grid(row=4, column=0, columnspan=2)
resumen_text = tk.Text(app, wrap="word", height=10, width=50)
resumen_text.grid(row=5, column=0, columnspan=2, padx=10, pady=10)
 
save_button = tk.Button(app, text="Guardar Resumen", command=save_summary)
save_button.grid(row=6, column=0, columnspan=2)
 
status_label = tk.Label(app, text="")
status_label.grid(row=7, column=0, columnspan=2)
 
app.mainloop()